"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  getDefaultOptions: true,
  withDefaultOptions: true,
  getExtensions: true,
  createCypherEditor: true
};
exports.createCypherEditor = createCypherEditor;
exports.withDefaultOptions = exports.getExtensions = exports.getDefaultOptions = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _autocomplete = require("@codemirror/autocomplete");
var _state = require("@codemirror/state");
var _view = require("@codemirror/view");
var _cypherCodemirrorBase = require("./cypher-codemirror-base");
Object.keys(_cypherCodemirrorBase).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _cypherCodemirrorBase[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _cypherCodemirrorBase[key];
    }
  });
});
var _cypherStateDefinitions = require("./cypher-state-definitions");
var _cypherStateSelectors = require("./cypher-state-selectors");
var _cypherExtensions = require("./cypher-extensions");
Object.keys(_cypherExtensions).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _cypherExtensions[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _cypherExtensions[key];
    }
  });
});
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
const getDefaultOptions = () => _objectSpread({}, defaultOptions);
exports.getDefaultOptions = getDefaultOptions;
const withDefaultOptions = options => {
  const combinedOptions = _objectSpread({}, defaultOptions);
  for (let key of Object.keys(options)) {
    if (options[key] !== undefined) {
      combinedOptions[key] = options[key];
    }
  }
  return combinedOptions;
};
exports.withDefaultOptions = withDefaultOptions;
const getExtensions = (options = {}, {
  lintConf = new _state.Compartment(),
  autocompleteConf = new _state.Compartment(),
  readableConf = new _state.Compartment(),
  readOnlyConf = new _state.Compartment(),
  showLinesConf = new _state.Compartment(),
  lineWrappingConf = new _state.Compartment(),
  historyConf = new _state.Compartment(),
  placeholderConf = new _state.Compartment(),
  tabKeyConf = new _state.Compartment(),
  themeConf = new _state.Compartment(),
  tooltipAbsoluteConf = new _state.Compartment(),
  onLineNumberClick = () => {},
  onFocusChanged = () => {},
  onScrollChanged = () => {},
  onKeyDown = () => {}
} = {}) => {
  const combinedOptions = withDefaultOptions(options);
  const {
    autocomplete,
    autocompleteCloseOnBlur,
    history,
    tabKey,
    lineNumberFormatter,
    lineNumbers,
    lineWrapping,
    lint,
    placeholder,
    readOnly,
    readOnlyCursor,
    theme,
    tooltipAbsolute
  } = combinedOptions;
  return [(0, _cypherExtensions.domListener)({
    onFocusChanged,
    onScrollChanged,
    onKeyDown
  }), (0, _cypherExtensions.cypherLanguage)(), lintConf.of((0, _cypherExtensions.getLintExtensions)({
    readOnly,
    lint
  })), autocompleteConf.of((0, _cypherExtensions.getAutocompleteExtensions)({
    readOnly,
    autocomplete,
    autocompleteCloseOnBlur
  })), showLinesConf.of((0, _cypherExtensions.getLineNumbersExtensions)({
    lineNumbers,
    lineNumberFormatter,
    onLineNumberClick
  })), lineWrappingConf.of((0, _cypherExtensions.getLineWrappingExtensions)({
    lineWrapping
  })), historyConf.of((0, _cypherExtensions.getHistoryExtensions)({
    history
  })), tabKeyConf.of((0, _cypherExtensions.getTabKeyExtensions)({
    tabKey
  })), readableConf.of((0, _cypherExtensions.getReadableExtensions)({
    readOnly,
    readOnlyCursor
  })), placeholderConf.of((0, _cypherExtensions.getPlaceholderExtensions)({
    placeholder
  })), _cypherExtensions.syntaxCSS, themeConf.of((0, _cypherExtensions.getThemeExtensions)({
    theme
  })), tooltipAbsoluteConf.of((0, _cypherExtensions.getTooltipAbsoluteExtensions)({
    tooltipAbsolute
  })), readOnlyConf.of((0, _cypherExtensions.getReadOnlyExtensions)({
    readOnly,
    readOnlyCursor
  }))];
};
exports.getExtensions = getExtensions;
const defaultOptions = _objectSpread(_objectSpread({}, _cypherCodemirrorBase.defaultOptions), {}, {
  preExtensions: [],
  postExtensions: []
});
function createCypherEditor(parentDOMElement, options = {}) {
  const combinedOptions = withDefaultOptions(options);
  const {
    autofocus,
    position,
    parseOnSetValue,
    schema,
    value,
    preExtensions,
    postExtensions
  } = combinedOptions;
  let {
    autocomplete,
    autocompleteOpen,
    autocompleteCloseOnBlur,
    autocompleteTriggerStrings,
    history,
    lineNumberFormatter,
    lineNumbers,
    lineWrapping,
    lint,
    placeholder,
    readOnly,
    readOnlyCursor,
    tabKey,
    tooltipAbsolute
  } = combinedOptions;
  const {
    on: onValueChanged,
    off: offValueChanged,
    fire: fireValueChanged
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const {
    on: onFocusChanged,
    off: offFocusChanged,
    fire: fireFocusChanged
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const {
    on: onScrollChanged,
    off: offScrollChanged,
    fire: fireScrollChanged
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const {
    on: onPositionChanged,
    off: offPositionChanged,
    fire: firePositionChanged
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const {
    on: onAutocompleteChanged,
    off: offAutocompleteChanged,
    fire: fireAutocompleteChanged
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const {
    on: onLineNumberClick,
    off: offLineNumberClick,
    fire: fireLineNumberClick
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const {
    on: onKeyDown,
    off: offKeyDown,
    fire: fireKeyDown
  } = (0, _cypherCodemirrorBase.createEventHandlers)();
  const lineNumberClick = (line, event) => {
    fireLineNumberClick(line, event);
  };
  const keyDown = event => {
    fireKeyDown(event);
  };
  const positionChanged = positionObject => {
    firePositionChanged((0, _cypherCodemirrorBase.positionOldToNew)(positionObject));
  };
  const autocompleteChanged = (newAutocompleteOpen, from, options) => {
    autocompleteOpen = newAutocompleteOpen;
    fireAutocompleteChanged(autocompleteOpen, from, options);
  };
  const focusChanged = focused => {
    fireFocusChanged(focused);
  };
  const scrollChanged = scrollInfo => {
    fireScrollChanged(scrollInfo);
  };
  const updateListener = _view.EditorView.updateListener.of(v => {
    if (v.docChanged) {
      valueChanged((0, _cypherStateSelectors.getStateValue)(v.state), v.changes);
      positionChanged((0, _cypherStateSelectors.getStatePosition)(v.state));
    } else if (v.selectionSet) {
      const oldPosition = (0, _cypherStateSelectors.getStatePositionAbsolute)(v.startState);
      const newPosition = (0, _cypherStateSelectors.getStatePositionAbsolute)(v.state);
      if (oldPosition !== newPosition) {
        positionChanged((0, _cypherStateSelectors.getStatePosition)(v.state));
      }
      if (deferredAutocomplete) {
        deferredAutocomplete = false;
        showAutocomplete();
      }
    }
    const startStatus = (0, _autocomplete.completionStatus)(v.startState);
    const endStatus = (0, _autocomplete.completionStatus)(v.state);
    if (startStatus !== "active" && endStatus === "active") {
      const {
        transactions
      } = v;
      const autocompleteResults = [];
      for (let transaction of transactions) {
        const {
          effects
        } = transaction;
        if (effects) {
          for (let effect of effects) {
            const {
              value: values
            } = effect;
            if (values) {
              for (let value of values) {
                const {
                  result
                } = value;
                if (result && typeof result === "object") {
                  const {
                    from,
                    options
                  } = result;
                  if (from !== undefined && options !== undefined) {
                    autocompleteResults.push({
                      from,
                      options
                    });
                  }
                }
              }
            }
          }
        }
      }
      if (autocompleteResults.length > 0) {
        if (autocompleteResults.length > 1) {
          console.error("multiple autocomplete results found in update transactions");
        }
        const {
          from,
          options
        } = autocompleteResults[0];
        autocompleteChanged(true, from, options);
      }
    } else if (startStatus !== null && endStatus === null) {
      autocompleteChanged(false);
    }
  });
  const preConf = new _state.Compartment();
  const lintConf = new _state.Compartment();
  const autocompleteConf = new _state.Compartment();
  const readableConf = new _state.Compartment();
  const readOnlyConf = new _state.Compartment();
  const showLinesConf = new _state.Compartment();
  const lineWrappingConf = new _state.Compartment();
  const historyConf = new _state.Compartment();
  const placeholderConf = new _state.Compartment();
  const tabKeyConf = new _state.Compartment();
  const themeConf = new _state.Compartment();
  const tooltipAbsoluteConf = new _state.Compartment();
  const postConf = new _state.Compartment();
  const initialState = _state.EditorState.create({
    doc: value,
    extensions: [preConf.of(preExtensions), ...getExtensions(combinedOptions, {
      lintConf,
      autocompleteConf,
      tabKeyConf,
      readableConf,
      readOnlyConf,
      showLinesConf,
      lineWrappingConf,
      historyConf,
      placeholderConf,
      themeConf,
      tooltipAbsoluteConf,
      postConf,
      onLineNumberClick: lineNumberClick,
      onFocusChanged: focusChanged,
      onScrollChanged: scrollChanged,
      onKeyDown: keyDown
    }), updateListener, postConf.of(postExtensions)]
  });
  let editor = new _view.EditorView({
    parent: parentDOMElement,
    state: initialState
  });
  editor.version = 1;
  editor.newContentVersion = function newContentVersion() {
    this.version += 1;
    return this.version;
  };
  editor.newContentVersion.bind(editor);
  editor.dispatch({
    effects: [_cypherStateDefinitions.initEditorSupportEffect]
  });
  const editorSupport = (0, _cypherStateSelectors.getStateEditorSupport)(editor.state);
  editorSupport.update(value);
  const getPositionForValue = positionValue => (0, _cypherStateSelectors.getStatePositionForAny)(editor.state, (0, _cypherCodemirrorBase.positionNewToOld)(positionValue));
  const setPosition = (positionParam, scrollIntoView = true) => {
    const positionObject = getPositionForValue(positionParam);
    if (positionObject) {
      const {
        position
      } = positionObject;
      editor.dispatch(editor.state.update({
        scrollIntoView,
        selection: {
          anchor: position
        }
      }));
    }
  };
  let deferredAutocomplete = false;
  const showDeferredAutocomplete = () => {
    deferredAutocomplete = true;
    setTimeout(() => {
      if (deferredAutocomplete) {
        deferredAutocomplete = false;
        showAutocomplete();
      }
    }, 50);
  };
  const showAutocomplete = () => {
    (0, _autocomplete.startCompletion)(editor);
  };
  const hideAutocomplete = () => {
    (0, _autocomplete.closeCompletion)(editor);
  };
  if (position !== undefined) {
    setPosition(position);
  }
  if (schema !== undefined) {
    editorSupport.setSchema(schema);
  }
  if (autocompleteOpen === true) {
    showAutocomplete();
  }
  if (autofocus) {
    editor.contentDOM.focus();
  }
  const setPreExtensions = preExtensions => {
    editor.dispatch({
      effects: preConf.reconfigure(preExtensions)
    });
  };
  const setPostExtensions = postExtensions => {
    editor.dispatch({
      effects: postConf.reconfigure(postExtensions)
    });
  };
  const valueChanged = (value, changes) => {
    if (autocomplete && Array.isArray(autocompleteTriggerStrings)) {
      let changedText = [];
      changes.iterChanges((fromA, toA, fromB, toB, inserted) => {
        changedText = inserted.text;
      });
      if (changedText.length > 0 && changedText.length <= 2) {
        const text = changedText[0];
        if (autocompleteTriggerStrings.indexOf(text) !== -1) {
          showDeferredAutocomplete();
        } else if (changedText.length === 2) {
          const longerText = text + changedText[1];
          if (autocompleteTriggerStrings.indexOf(longerText) !== -1) {
            showDeferredAutocomplete();
          }
        }
      }
    }
    fireValueChanged(value, changes);
  };
  const setValue = (value = defaultOptions.value, parseOnSetValueParam = parseOnSetValue) => {
    const update = editor.state.update({
      changes: {
        from: 0,
        to: (0, _cypherStateSelectors.getStateLength)(editor.state),
        insert: value
      }
    });
    editor.update([update]);
    if (parseOnSetValueParam !== false) {
      const version = editor.newContentVersion();
      const editorSupport = (0, _cypherStateSelectors.getStateEditorSupport)(editor.state);
      editorSupport.update(value, version);
      (0, _cypherExtensions.fixColors)(editor, editorSupport);
    }
  };
  const selectAutocompleteOption = index => {
    editor.dispatch({
      effects: (0, _autocomplete.setSelectedCompletion)(index)
    });
    (0, _autocomplete.acceptCompletion)(editor);
  };
  const setHistory = (newHistory = defaultOptions.history) => {
    history = newHistory;
    editor.dispatch({
      effects: historyConf.reconfigure((0, _cypherExtensions.getHistoryExtensions)({
        history
      }))
    });
  };
  const clearHistory = () => {
    editor.dispatch({
      effects: historyConf.reconfigure([])
    });
    editor.dispatch({
      effects: historyConf.reconfigure((0, _cypherExtensions.getHistoryExtensions)({
        history
      }))
    });
  };
  const setLineNumbers = (newLineNumbers = defaultOptions.lineNumbers) => {
    lineNumbers = newLineNumbers;
    editor.dispatch({
      effects: showLinesConf.reconfigure((0, _cypherExtensions.getLineNumbersExtensions)({
        lineNumbers,
        lineNumberFormatter,
        onLineNumberClick: lineNumberClick
      }))
    });
  };
  const setLineNumberFormatter = (newLineNumberFormatter = defaultOptions.lineNumberFormatter) => {
    lineNumberFormatter = newLineNumberFormatter;
    editor.dispatch({
      effects: showLinesConf.reconfigure((0, _cypherExtensions.getLineNumbersExtensions)({
        lineNumbers,
        lineNumberFormatter,
        onLineNumberClick: lineNumberClick
      }))
    });
  };
  const setReadOnly = (newReadOnly = defaultOptions.readOnly) => {
    readOnly = newReadOnly;
    editor.dispatch({
      effects: [readableConf.reconfigure((0, _cypherExtensions.getReadableExtensions)({
        readOnly,
        readOnlyCursor
      })), readOnlyConf.reconfigure((0, _cypherExtensions.getReadOnlyExtensions)({
        readOnly,
        readOnlyCursor
      })), autocompleteConf.reconfigure((0, _cypherExtensions.getAutocompleteExtensions)({
        readOnly,
        autocomplete,
        autocompleteCloseOnBlur
      })), lintConf.reconfigure((0, _cypherExtensions.getLintExtensions)({
        readOnly,
        lint
      }))]
    });
  };
  const setReadOnlyCursor = (newReadOnlyCursor = defaultOptions.readOnlyCursor) => {
    readOnlyCursor = newReadOnlyCursor;
    editor.dispatch({
      effects: [readableConf.reconfigure((0, _cypherExtensions.getReadableExtensions)({
        readOnly,
        readOnlyCursor
      })), readOnlyConf.reconfigure((0, _cypherExtensions.getReadOnlyExtensions)({
        readOnly,
        readOnlyCursor
      }))]
    });
  };
  const setPlaceholder = (newPlaceholder = defaultOptions.placeholder) => {
    placeholder = newPlaceholder;
    editor.dispatch({
      effects: [placeholderConf.reconfigure((0, _cypherExtensions.getPlaceholderExtensions)({
        placeholder
      }))]
    });
  };
  const setLineWrapping = (newLineWrapping = defaultOptions.lineWrapping) => {
    lineWrapping = newLineWrapping;
    editor.dispatch({
      effects: [lineWrappingConf.reconfigure((0, _cypherExtensions.getLineWrappingExtensions)({
        lineWrapping
      }))]
    });
  };
  const setAutocomplete = (newAutocomplete = defaultOptions.autocomplete) => {
    autocomplete = newAutocomplete;
    editor.dispatch({
      effects: autocompleteConf.reconfigure((0, _cypherExtensions.getAutocompleteExtensions)({
        readOnly,
        autocomplete,
        autocompleteCloseOnBlur
      }))
    });
  };
  const setAutocompleteCloseOnBlur = (newAutocompleteCloseOnBlur = defaultOptions.autocompleteCloseOnBlur) => {
    autocompleteCloseOnBlur = newAutocompleteCloseOnBlur;
    editor.dispatch({
      effects: autocompleteConf.reconfigure((0, _cypherExtensions.getAutocompleteExtensions)({
        readOnly,
        autocomplete,
        autocompleteCloseOnBlur
      }))
    });
  };
  const setAutocompleteTriggerStrings = (newAutocompleteTriggerStrings = defaultOptions.autocompleteTriggerStrings) => {
    autocompleteTriggerStrings = newAutocompleteTriggerStrings;
  };
  const setLint = (newLint = defaultOptions.lint) => {
    lint = newLint;
    editor.dispatch({
      effects: lintConf.reconfigure((0, _cypherExtensions.getLintExtensions)({
        readOnly,
        lint
      }))
    });
  };
  const getPosition = () => {
    return (0, _cypherCodemirrorBase.positionOldToNew)((0, _cypherStateSelectors.getStatePosition)(editor.state));
  };
  const getLineCount = () => {
    return editor ? (0, _cypherStateSelectors.getStateLineCount)(editor.state) : 0;
  };
  const setSchema = (schema = defaultOptions.schema) => {
    editorSupport.setSchema(schema);
    if (autocompleteOpen) {
      showAutocomplete();
    }
  };
  const setTheme = (theme = defaultOptions.theme) => {
    editor.dispatch({
      effects: themeConf.reconfigure((0, _cypherExtensions.getThemeExtensions)({
        theme
      }))
    });
  };
  const setTooltipAbsolute = (newTooltipAbsolute = defaultOptions.tooltipAbsolute) => {
    tooltipAbsolute = newTooltipAbsolute;
    editor.dispatch({
      effects: tooltipAbsoluteConf.reconfigure((0, _cypherExtensions.getTooltipAbsoluteExtensions)({
        tooltipAbsolute
      }))
    });
  };
  const focus = () => {
    editor && editor.focus();
  };
  const destroy = () => {
    editor && editor.destroy();
  };
  const setAutocompleteOpen = (open = defaultOptions.autocompleteOpen) => {
    if (open) {
      showAutocomplete();
    } else {
      hideAutocomplete();
    }
  };
  const setTabKey = (newTabKey = defaultOptions.tabKey) => {
    tabKey = newTabKey;
    editor.dispatch({
      effects: tabKeyConf.reconfigure((0, _cypherExtensions.getTabKeyExtensions)({
        tabKey
      }))
    });
  };
  const editorAPI = {
    clearHistory,
    destroy,
    focus,
    getLineCount,
    getPosition,
    getPositionForValue,
    selectAutocompleteOption,
    setAutocomplete,
    setAutocompleteCloseOnBlur,
    setAutocompleteOpen,
    setAutocompleteTriggerStrings,
    setHistory,
    setLineNumberFormatter,
    setLineNumbers,
    setLineWrapping,
    setLint,
    setPlaceholder,
    setPosition,
    setReadOnly,
    setReadOnlyCursor,
    setSchema,
    setTabKey,
    setTheme,
    setTooltipAbsolute,
    setValue,
    onAutocompleteChanged,
    offAutocompleteChanged,
    onFocusChanged,
    offFocusChanged,
    onKeyDown,
    offKeyDown,
    onLineNumberClick,
    offLineNumberClick,
    onPositionChanged,
    offPositionChanged,
    onScrollChanged,
    offScrollChanged,
    onValueChanged,
    offValueChanged,
    setPreExtensions,
    setPostExtensions,
    codemirror: editor,
    editorSupport
  };
  if (parseOnSetValue !== false) {
    const version = editor.newContentVersion();
    editorSupport.update(value, version);
    (0, _cypherExtensions.fixColors)(editor, editorSupport);
  }
  return {
    editor: editorAPI
  };
}